package it.eng.configurazione.service;

import java.nio.channels.ConnectionPendingException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.hibernate.Query;
import org.hibernate.Session;

import it.eng.configurazione.bean.ConnetionParameter;
import it.eng.configurazione.bean.DataSource;
import it.eng.configurazione.bean.ExtractionProcess;
import it.eng.configurazione.bean.Field;
import it.eng.configurazione.bean.Interface;
import it.eng.configurazione.bean.Kpi;
import it.eng.configurazione.bean.Operation;
import it.eng.configurazione.bean.OperationParameter;
import it.eng.configurazione.bean.Project;
import it.eng.configurazione.bean.ProjectKpi;
import it.eng.configurazione.bean.SourceType;

public class ConfigurationDAO {
	Session sessione= null;
	
	public ConfigurationDAO(Session sessione){
		this.sessione=sessione;
	}
	
	public ConfigurationDAO(){
		this.sessione=HibernateUtils.getSession();
	}	
	
	
	public Interface readInterface(String name){
		Interface interfaccia=null;
		sessione.beginTransaction();
		Query query=sessione.createQuery("from Interface where name=?");
		query.setParameter(0, name);
		List<Interface> risultato=query.list();
		if (risultato.size()==1) interfaccia= risultato.get(0);
		sessione.getTransaction().commit();
		return interfaccia;
	}

	public Operation readOperation(String name){
		Operation process=null;
		sessione.beginTransaction();
		Query query=sessione.createQuery("from Operation where name=?");
		query.setParameter(0, name);
		List<Operation> risultato=query.list();
		if (risultato.size()==1) process= risultato.get(0);
		
		sessione.getTransaction().commit();
		return process;
	}	
	
	public ExtractionProcess readExtractionProcess(String name){
		ExtractionProcess process=null;
		sessione.beginTransaction();
		Query query=sessione.createQuery("from ExtractionProcess where name=?");
		query.setParameter(0, name);
		List<ExtractionProcess> risultato=query.list();
		if (risultato.size()==1) process= risultato.get(0);
		
		sessione.getTransaction().commit();
		return process;
	}	
	
	/**
	 * estrago i progetti legati all'aspetto 
	 * @param name
	 * @return
	 */
	public List<Kpi> readKpi(String name){
		sessione.beginTransaction();
		Query query=sessione.createQuery("select k from Kpi as k inner join k.measurableAttributes as ms where ms.tableName=?");
		query.setParameter(0, name);
		List<Kpi> projects=query.list();
		sessione.getTransaction().commit();
		return projects;
	}	
	
	/**
	 * Dato il nome della tabella dei fatti restituisce l'elenco dei progetti
	 * legati a quel fatto.
	 * Es.: List<String> tmp=dao.readProject("FT_REQUIREMENTS");
	 * @param name
	 * @return
	 */
	public List<String> readProject(String name){
		sessione.beginTransaction();
		Query query=sessione.createQuery("select distinct p.pk.project.name from ProjectKpi as p right outer join p.pk.kpi as k where k.name in (select distinct k.name from Kpi as k right outer join k.measurableAttributes as ms where ms.tableName=?)");
		query.setParameter(0, name);
		List<String> projects=query.list();
		sessione.getTransaction().commit();
		return projects;
	}	
	
	/**
	 *  restituisce l'elenco dei progetti
	 * @param name
	 * @return
	 */
	public List<String> readAllProject(){
		sessione.beginTransaction();
		Query query=sessione.createQuery("select distinct p.name from Project as p");
		List<String> projects=query.list();
		sessione.getTransaction().commit();
		return projects;
	}		
	
	public DataSource readDataSource(String name){
		DataSource ds=null;
		sessione.beginTransaction();
		Query query=sessione.createQuery("from DataSource where name=?");
		query.setParameter(0, name);
		List<DataSource> risultato=query.list();
		if (risultato.size()==1) ds= risultato.get(0);
		sessione.getTransaction().commit();
		return ds;
	}		
	
	public void read(){
		
		Operation operazione1=null;
		
		sessione.beginTransaction();
		operazione1= (Operation)sessione.load(Operation.class, 1);
			
		sessione.getTransaction().commit();
			
	}
	
	public void insert(){
		
		Interface interfaccia1=new Interface();
		interfaccia1.setClassforname("classe");
		interfaccia1.setName("INT_1");
		interfaccia1.setDescription("DESCRIZIONE_INT_1");
		interfaccia1.setXmlSchema("XML_SCHEMA");
		
		ExtractionProcess processo1=new ExtractionProcess();
		processo1.setInterfaceInfo(interfaccia1);
		processo1.setName("PROCESSO_1");
		processo1.setDescription("Descrizione processo1");
		//processo1.addOperation(p);
		


		
		ConnetionParameter conP1=new ConnetionParameter();
		//conP1.setId(1);
		conP1.setName("utente");
		conP1.setValue("pippo");
		ConnetionParameter conP2=new ConnetionParameter();
		//conP2.setId(2);
		conP2.setName("utente");
		conP2.setValue("pippo");		
		
		SourceType tipo1=new SourceType();
		//tipo1.setId(1);
		tipo1.setName("TIPO1");
		tipo1.setDescription("DESCRIZIONE");
		tipo1.setClassforname("it.eng.ciccio.class");
		
		DataSource ds=new DataSource();
		ds.setId(1);
		ds.setDescription("DDD");
		ds.setName("NOME");
		ds.setSource(tipo1);
		ds.addParameter(conP1);
		ds.addParameter(conP2);
		//conP1.setDataSource(ds);
		//conP2.setDataSource(ds);

		DataSource ds2=new DataSource();
		ds2.setId(2);
		ds2.setDescription("DDD2");
		ds2.setName("NOME2");
		ds2.setSource(tipo1);		

		
		Operation operazione1=new Operation();
		operazione1.addProcess(processo1);
		operazione1.setDataSource(ds);
		//processo1.addOperation(operazione1);
		
		
		Field campo1=new Field();
		campo1.setInterfacePosition("1");
		campo1.setMappingScript("Script groovy");
		campo1.setType("Stringa");
		Field campo2=new Field();
		campo2.setInterfacePosition("1");
		campo2.setMappingScript("Script groovy");
		campo2.setType("Stringa");
		
		OperationParameter operPar1=new OperationParameter();
		operPar1.setName("PAR_1");
		operPar1.setValue("VALORE_1");
		OperationParameter operPar2=new OperationParameter();
		operPar2.setName("PAR_2");
		operPar2.setValue("VALORE_2");
		
		operazione1.addField(campo1);
		operazione1.addField(campo2);
		operazione1.addParameter(operPar1);
		operazione1.addParameter(operPar2);
		
		sessione.beginTransaction();
		sessione.save(interfaccia1);
		sessione.save(processo1);
		
		sessione.save(tipo1);

		sessione.save(ds);		
		sessione.save(ds2);
		sessione.save(operazione1);
		
		sessione.getTransaction().commit();
	
	}
	
}
